# -*- coding: utf-8 -*-
"""
Created on Tue Nov 19 12:59:34 2024

@author: mstep
"""

import pygame

class BallSprite:
    def __init__(self, the_app, center, radius, velocity, color):
        '''
        Save the ball's parameters.
        center - A tuple giving the X and Y coordinates of the ball's center.
        velocity - A tuple giving the ball's speeds in pixels per second in
            the X and Y directions.
        '''
        self.the_app = the_app
        self.center = center
        self.radius = radius
        self.velocity = velocity
        self.color = color

    def update(self, elapsed_seconds, bounds):
        '''Update the ball's position.'''
        # Calculate the min and max X value the ball's center can have.
        xmin = bounds.left + self.radius
        xmax = bounds.right - self.radius

        # Get the current velocity components.
        vx = self.velocity[0]
        vy = self.velocity[1]

        # Update the ball's X coordinate.
        x = self.center[0] + self.velocity[0] * elapsed_seconds
        if x < xmin:
            self.the_app.sprite_bounced()
            x = xmin + (xmin - x)
            vx = -vx
        elif x >= xmax:
            self.the_app.sprite_bounced()
            x = xmax - (x - xmax)
            vx = -vx

        # Calculate the min and max Y value the ball's center can have.
        ymin = bounds.top + self.radius
        ymax = bounds.bottom - self.radius

        # Update the ball's Y coordinate.
        y = self.center[1] + self.velocity[1] * elapsed_seconds
        if y < ymin:
            self.the_app.sprite_bounced()
            y = ymin + (ymin - y)
            vy = -vy
        elif y >= ymax:
            self.the_app.sprite_bounced()
            y = ymax - (y - ymax)
            vy = -vy

        # Update the ball's velocity and position.
        self.velocity = (vx, vy)
        self.center = (x, y)

        # Return True to indicate this sprite is still alive.
        return True

    def draw(self, surface):
        '''Draw the ball.'''
        pygame.draw.circle(surface, self.color, self.center, self.radius)

#%%

import pygame
import random

def random_color():
    colors = [
        pygame.Color('red'),
        pygame.Color('green'),
        pygame.Color('blue'),
        pygame.Color('yellow'),
        pygame.Color('orange'),
        pygame.Color('cyan'),
        pygame.Color('magenta'),
    ]
    return random.choice(colors)

#%%

import pygame
import pygame.locals
import random

class BounceApp:
    def __init__(self, game_name, screen_size, max_fps, bg_color,
                 num_sprites, min_radius, max_radius,
                 min_velocity, max_velocity):
        # Save game parameters.
        self.game_name = game_name
        self.screen_size = screen_size
        self.max_fps = max_fps
        self.bg_color = bg_color

        # Initialize pygame.
        pygame.init()
        pygame.display.set_caption(f'{self.game_name}')
        self.surface = pygame.display.set_mode(self.screen_size)
        self.bounds = self.surface.get_rect()

        # Prepare sounds.
        self.boing_sound = pygame.mixer.Sound('boing.wav')

        # Make initial sprites.
        self.make_sprites(num_sprites, min_radius, max_radius,
                          min_velocity, max_velocity)

        # Start the event loop.
        self.event_loop()

    def sprite_bounced(self):
        '''Play the sprite bounced sound.'''
        self.boing_sound.play()

    def make_sprites(self, num_sprites, min_radius, max_radius,
                     min_velocity, max_velocity):
        '''Make the sprites.'''
        self.all_sprites = []
        for _ in range(num_sprites):
            radius = random.randrange(min_radius, max_radius)

            min_x = self.bounds.left + radius
            max_x = self.bounds.right - radius
            x = random.randint(min_x, max_x)

            min_y = self.bounds.top + radius
            max_y = self.bounds.bottom - radius
            y = random.randint(min_y, max_y)
            center = (x, y)

            vx = random.randint(min_velocity, max_velocity)
            if random.randint(0, 1) == 0:
                vx = -vx
            vy = random.randint(min_velocity, max_velocity)
            if random.randint(0, 1) == 0:
                vy = -vy
            velocity = (vx, vy)

            self.all_sprites.append(
                BallSprite(self, center, radius, velocity, random_color()))

    # Process events.
    def event_loop(self):
        clock = pygame.time.Clock()
        running = True
        while running:
            # Event loop.
            for event in pygame.event.get():
                if event.type == pygame.locals.QUIT:
                    running = False
                    break

            # See how much time has passed since the last update.
            elapsed_ticks = clock.tick(self.max_fps)
            elapsed_seconds = elapsed_ticks / 1000

            # Update the sprites.
            self.update_sprites(elapsed_seconds)

            # Draw the sprites.
            self.draw_sprites()

            # Update the display.
            pygame.display.update()

        # End the program.
        pygame.display.quit()
        pygame.quit()

    def update_sprites(self, elapsed_seconds):
        '''Let the sprites update themselves.'''
        living_sprites = []
        for sprite in self.all_sprites:
            # The sprite.update function returns True if the sprite is still alive.
            if sprite.update(elapsed_seconds, self.bounds):
                # This sprite is still alive.
                living_sprites.append(sprite)

        # Save the living sprites.
        self.all_sprites = living_sprites

    # Draw the sprites.
    def draw_sprites(self):
        self.surface.fill(self.bg_color)
        for sprite in self.all_sprites:
            sprite.draw(self.surface)


# Main program.
app = BounceApp('Bouncing Balls', screen_size=(800, 600), max_fps=30,
    bg_color=pygame.Color('cadet blue 3'), num_sprites=10,
    min_radius=20, max_radius=50, min_velocity=30, max_velocity=200)
print('Done')
